#include "puissance4.h"

void print_usage(const char* program_name) {
    printf("Usage: %s <mode> <row> <col>\n", program_name);
    printf("%*smode  specifies the mode: 1 = single player game (random),\n", 5, "");
    printf("%*s      2 = single player game (AI), 3 = two players game\n", 5, "");
    printf("%*srow   specifies the number of rows (>= %d)\n", 5, "", MIN_ROW);
    printf("%*scol   specifies the number of columns (>= %d)\n", 5, "", MIN_COLUMN);
}

bool validate_arguments(char const* argv[], int* game_mode, int* row_count, int* col_count) {
    *game_mode = atoi(argv[1]);
    *row_count = atoi(argv[2]);
    *col_count = atoi(argv[3]);

    if (!is_valid_game_mode(*game_mode) || !is_valid_board_size(*row_count, *col_count)) {
        print_usage(argv[0]);
        return false;
    }

    return true;
}

int main(int argc, char const* argv[]) {
    if (argc != 4) {
        print_usage(argv[0]);
        return EXIT_FAILURE;
    }

    int game_mode, row_count, col_count;
    if (!validate_arguments(argv, &game_mode, &row_count, &col_count)) {
        return EXIT_FAILURE;
    }

    srand(AI_SEED);
    int** board = create_board(row_count, col_count);
    if (board == NULL) {
        fprintf(stderr, "Error: Unable to allocate memory for the board.\n");
        return EXIT_FAILURE;
    }

    fill_board(row_count, col_count, board);
    printf("Board size is %dx%d (rows x col)", row_count, col_count);
    print_board(row_count, col_count, board);

    int current_turn = 0;
    struct coordinate empty_slot;

    while (true) {
        // selected_column starts at 0
        int selected_column = get_selected_column(game_mode, current_turn, col_count, row_count, board);
        if (!is_valid_column(selected_column, col_count)) {
            printf("\nInvalid column number. Please try again.\n");
            continue;
        }

        empty_slot = find_empty_slot(selected_column, row_count, board);
        place_token(current_turn % 2, empty_slot, board);
        print_board(row_count, col_count, board);

        if (does_player_win(empty_slot, row_count, col_count, board)) {
            announce_winner(game_mode, current_turn);
            break;
        }

        if (is_board_full(current_turn, row_count, col_count)) {
            printf("\nIt is a draw.\n");
            break;
        }

        current_turn++;
    }

    if (destroy_board(row_count, board) == EXIT_FAILURE) {
        fprintf(stderr, "Error: Unable to free memory for the board.\n");
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

void announce_winner(enum GAME_MODE game_mode, int current_turn) {
    char* winner = (current_turn % 2 == 0) ? "Player one" : "Computer";

    if (game_mode == PLAYER_VS_PLAYER) {
        printf("\nPlayer %s won!\n", (current_turn % 2 == 0) ? "one" : "two");
    } else {
        printf("\n%s won!\n", winner);
    }
}

bool is_valid_game_mode(int game_mode) {
    return game_mode >= 1 && game_mode <= 3;
}

bool is_valid_board_size(int rows, int cols) {
    return rows >= MIN_ROW && cols >= MIN_COLUMN;
}
