#ifndef _GAME_H_
#define _GAME_H_

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include "../board/board.h"
#include "../computer/computer.h"

enum GAME_MODE {
    PLAYER_VS_RANDOM_AI = 1,
    PLAYER_VS_SMART_AI = 2,
    PLAYER_VS_PLAYER = 3
};

struct coordinate {
    int x;
    int y;
};

/**
 * @brief Finds the lowest empty slot in the specified column.
 *
 * @param x The column in which to look: indexed from 0.
 * @param rows The total number of rows in the board.
 * @param board The game board.
 * @return A coordinate representing the empty slot.
 */
struct coordinate find_empty_slot(int x, int rows, int** board);

/**
 * @brief Checks whether the player has won after placing a token.
 *
 * @param coordinate The position of the last placed token.
 * @param rows The number of rows in the board.
 * @param cols The number of columns in the board.
 * @param board The game board.
 * @return true if the player has won, false otherwise.
 */
bool does_player_win(struct coordinate coordinate, int rows, int cols, int** board);

/**
 * @brief Places a token on the board for the specified player.
 *
 * @param player The player number.
 * @param coordinate The position to place the token.
 * @param board The game board.
 */
void place_token(int player, struct coordinate coordinate, int** board);

/**
 * @brief Checks if the given column index is valid for the board.
 *
 * @param column The column index to check. starting from 0.
 * @param col_count The total number of columns in the board.
 * @return true if the column index is valid, false otherwise.
 */
bool is_valid_column(int column, int col_count);

/**
 * @brief Checks whether the board is full.
 *
 * @param current_turn The current player's turn.
 * @param row_count The number of rows.
 * @param col_count The number of columns.
 * @return true if the board is full, false otherwise.
 */
bool is_board_full(int current_turn, int row_count, int col_count);

/**
 * @brief Gets the column selected by the current player or AI.
 *
 * @param game_mode The current game mode.
 * @param current_turn The current turn.
 * @param col_count The number of columns.
 * @param row_count The number of rows.
 * @param board The game board.
 * @return The selected column index.
 */
int get_selected_column(enum GAME_MODE game_mode, int current_turn, int col_count, int row_count, int** board);

/**
 * @brief Determines if the current turn belongs to a human player.
 *
 * @param game_mode The current game mode.
 * @param current_turn The current turn.
 * @return true if it's a human's turn, false otherwise.
 */
bool is_human_turn(enum GAME_MODE game_mode, int current_turn);

/**
 * @brief Gets the move decided by the AI based on the game mode.
 *
 * @param game_mode The current game mode.
 * @param col_count The number of columns.
 * @param row_count The number of rows.
 * @param board The game board.
 * @return The selected column index.
 */
int get_ai_move(enum GAME_MODE game_mode, int col_count, int row_count, int** board);

/**
 * @brief Gets column input from the player and adjusts for 0-based indexing.
 *
 * @return The column number entered by the player.
 */
int get_player_input();

#endif
